--Increasing stats by skill
local MOD = {}

local version = 2

local str_skills = { "mechanics", "swimming", "bashing", "cutting", "melee", "throw", "driving", "survival", "tailor", "traps", "dodge", "stabbing", "unarmed", "barter", "computer", "cooking", "electronics", "fabrication", "firstaid", "speech", "archery", "gun", "launcher", "pistol", "rifle", "shotgun", "smg" }

mods["StatsThroughSkillsSTR"] = MOD

function MOD.on_new_player_created()
    game.add_msg("あなたのニューロンの一つ一つが、筋肉を学びたがっている")
    player:set_value("StatsThroughSkillsSTR", tostring(version))
    calculate_bonuses()
end

function MOD.on_skill_increased()
    calculate_bonuses()
end

function calculate_bonuses()
    game.add_msg("筋肉は地道な積み重ねにあり")

    handle_previous_version()

    local prev_str = player.str_max

    remove_existing_bonuses()

    local str_bonus = calc_bonus(str_skills)

    player:set_value("str_bonus", tostring(str_bonus))

    player.str_max = player.str_max + str_bonus

    print_results(player.str_max,"Str",prev_str)

    local hp_prev = {}
    for k,v in pairs(enums.hp_part) do
        if string.match(v, "num_hp_parts") == nil then
            hp_prev[v] = player:get_hp(v)
        end
    end

    player:recalc_hp()

    local hp_new = {}
    for k,v in pairs(enums.hp_part) do
        if string.match(v, "num_hp_parts") == nil then
            hp_new[v] = player:get_hp(v)
        end
    end

    for k,v in pairs(enums.hp_part) do
        if string.match(v, "num_hp_parts") == nil then
            if hp_prev[v] ~= hp_new[v] then
                player:heal(v, math.abs(hp_prev[v] - hp_new[v]))
            end
        end
    end

end

function remove_existing_bonuses()
    local str_bonus = tonumber(player:get_value("str_bonus"))

    if(str_bonus) then player.str_max = player.str_max - str_bonus end
end

function calc_bonus(skills_set)
    local skill_total = 0
    for _,s in ipairs(skills_set) do
        skill_total = skill_total + player:get_skill_level(skill_id(s))
    end

    return (skill_total > 3 and math.floor(math.pow((skill_total - 3), (1 / 1.96))) or 0)
end

function print_results(cur_stat,stat,prev_stat)
    if (prev_stat < cur_stat) then
        game.add_msg("熱心に筋肉を学び、その結果が反映された")
    elseif (prev_stat > cur_stat) then
        game.add_msg("筋肉が少し落ちた。だが、また鍛えれば良い")
    end
end

function handle_previous_version()
    local loaded_version = tonumber(player:get_value("StatsThroughSkillsSTR"))
    local character_eligible_for_migration = false
    local calendar = game.get_calendar_turn()
    if( function_exists( "calendar.days" ) ) then
        character_eligible_for_migration = calendar:days() >= 1
    else
        character_eligible_for_migration = calendar:years() >= 1
    end
    if( (not loaded_version or loaded_version < 2) and character_eligible_for_migration ) then --handle upgrading from original StatsThroughSkills to version 2
        game.add_msg("Migrating from version 1")

        local str_bonus = get_stat_bonus_for_skills_version_1(str_skills)

        player.str_max = player.str_max - str_bonus

        player:set_value("StatsThroughSkillsSTR", tostring(2))
    end
end

function get_stat_bonus_for_skills_version_1(skill_set)
    local total_bonus = 0
    for _,v in ipairs(skill_set) do
        local skill_level = player:get_skill_level(skill_id(v))
        local stat_bonus = 0
        if (skill_level / 3 < 3) then
            stat_bonus = stat_bonus + skill_level / 3
        else
            stat_bonus = stat_bonus + 3
        end
        total_bonus = total_bonus + math.floor(stat_bonus)
    end
    return total_bonus
end
